// WidgView.cpp : implementation of the CWidgetView class
//

#include "stdafx.h"
#include "Widget.h"
#include "WidgBase.h"
#include "Circle.h"
#include "Triangle.h"
#include "Square.h"
#include "WidgDoc.h"
#include "WidgView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWidgetView

IMPLEMENT_DYNCREATE(CWidgetView, CScrollView)

BEGIN_MESSAGE_MAP(CWidgetView, CScrollView)
    //{{AFX_MSG_MAP(CWidgetView)
    ON_WM_LBUTTONDOWN()
    ON_WM_CREATE()
    //}}AFX_MSG_MAP
    // Standard printing commands
    ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
    ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
    ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWidgetView construction/destruction

CWidgetView::CWidgetView()
{
}

CWidgetView::~CWidgetView()
{
}

BOOL CWidgetView::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CWidgetView drawing

void CWidgetView::OnDraw(CDC* pDC)
{
    CWidgetDoc* pDoc = GetDocument();
    ASSERT_VALID(pDoc);

    int nCount = pDoc->GetWidgetCount ();
    if (nCount) {
        for (int i=0; i<nCount; i++)
            pDoc->GetWidget (i)->Draw (pDC);
    }
}

void CWidgetView::OnInitialUpdate()
{
    CScrollView::OnInitialUpdate();

    CSize sizeTotal;
    sizeTotal.cx = sizeTotal.cy = 1024;
    SetScrollSizes(MM_TEXT, sizeTotal);

    m_nWidget = -1;     // Index of the widget that's being dragged
    m_pWidget = NULL;   // Pointer to temporary widget used in drag imaging
}

/////////////////////////////////////////////////////////////////////////////
// CWidgetView printing

BOOL CWidgetView::OnPreparePrinting(CPrintInfo* pInfo)
{
    // default preparation
    return DoPreparePrinting(pInfo);
}

void CWidgetView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
    // TODO: add extra initialization before printing
}

void CWidgetView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
    // TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CWidgetView diagnostics

#ifdef _DEBUG
void CWidgetView::AssertValid() const
{
    CScrollView::AssertValid();
}

void CWidgetView::Dump(CDumpContext& dc) const
{
    CScrollView::Dump(dc);
}

CWidgetDoc* CWidgetView::GetDocument() // non-debug version is inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CWidgetDoc)));
    return (CWidgetDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWidgetView message handlers

void CWidgetView::OnLButtonDown(UINT nFlags, CPoint point) 
{
    CWidgetDoc* pDoc = GetDocument ();
    int nCount = pDoc->GetWidgetCount ();

    if (nCount) {
        //
        // Convert the click point to logical coordinates.
        //
        CClientDC dc (this);
        OnPrepareDC (&dc);
        dc.DPtoLP (&point);

        //
        // Find out whether a widget was clicked.
        //
        int i;
        BOOL bHit = FALSE;
        for (i=nCount - 1; i>=0 && !bHit; i--) {
            CWidget* pWidget = pDoc->GetWidget (i);
            if (pWidget->PtInWidget (point)) {
                bHit = TRUE;
            }
        }

        //
        // If a widget was clicked, initiate a drag-drop operation.
        //
        if (bHit) {
            m_nWidget = ++i;

            HANDLE hData =
                ::GlobalAlloc (GMEM_MOVEABLE | GMEM_SHARE, sizeof (WIDGETDATA));

            WIDGETDATA* pWidgetData = (WIDGETDATA*) ::GlobalLock (hData);
            CWidget* pWidget = pDoc->GetWidget (m_nWidget);
            pWidgetData->nType = pWidget->GetType ();
            pWidget->GetItemRect (&pWidgetData->rcItem);
            pWidgetData->ptDrag = point;
            ::GlobalUnlock (hData);

            COleDataSource ods;
            UINT nFormat = ((CWidgetApp*) AfxGetApp ())->m_nFormat;
            ods.CacheGlobalData (nFormat, hData);

            DROPEFFECT de = ods.DoDragDrop (DROPEFFECT_COPY | DROPEFFECT_MOVE);

            if (de == DROPEFFECT_MOVE) {
                pDoc->RemoveWidget (m_nWidget);
                pDoc->UpdateAllViews (NULL);
            }
            m_nWidget = -1;
        }
    }
    CScrollView::OnLButtonDown(nFlags, point);
}

DROPEFFECT CWidgetView::OnDragEnter(COleDataObject* pDataObject,
    DWORD dwKeyState, CPoint point) 
{
    CScrollView::OnDragEnter(pDataObject, dwKeyState, point);

    //
    // Return now if the object being dragged is not a widget.
    //
    UINT nFormat = ((CWidgetApp*) AfxGetApp ())->m_nFormat;
    if (!pDataObject->IsDataAvailable (nFormat))
        return DROPEFFECT_NONE;

    //
    // Create a temporary widget for drag imaging.
    //
    HGLOBAL hData = pDataObject->GetGlobalData (nFormat);

    WIDGETDATA* pWidgetData = (WIDGETDATA*) ::GlobalLock (hData);
    int nType = pWidgetData->nType;
    CRect rect = pWidgetData->rcItem;
    CPoint pt = pWidgetData->ptDrag;
    ::GlobalUnlock (hData);

    switch (nType) {

    case 0: // Circle
        m_pWidget = new CCircle (rect);
        break;

    case 1: // Triangle
        m_pWidget = new CTriangle (rect);
        break;

    case 2: // Square
        m_pWidget = new CSquare (rect);
        break;

    default: // Just in case
        return DROPEFFECT_NONE;
    }

    //
    // Begin dragging.
    //
    CClientDC dc (this);
    OnPrepareDC (&dc);
    dc.DPtoLP (&point);

    m_sizeDelta.cx = pt.x - rect.left;
    m_sizeDelta.cy = pt.y - rect.top;
    CPoint ptDrag (point.x - m_sizeDelta.cx, point.y - m_sizeDelta.cy);
    m_ptOldImage.x = m_ptOldImage.y = -32000;
    m_ptPrevPos = point;

    return (dwKeyState & MK_CONTROL) ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
}

DROPEFFECT CWidgetView::OnDragOver(COleDataObject* pDataObject,
    DWORD dwKeyState, CPoint point) 
{
    CScrollView::OnDragOver(pDataObject, dwKeyState, point);

    //
    // Return now if the object being dragged is not a widget.
    //
    if (m_pWidget == NULL)
        return DROPEFFECT_NONE;

    //
    // Erase the old drag image and draw a new one if the cursor has moved.
    //
    CClientDC dc (this);
    OnPrepareDC (&dc);
    dc.DPtoLP (&point);

    if (point != m_ptPrevPos) {
        m_pWidget->DrawDragImage (&dc, m_ptOldImage);
        CPoint ptDrag (point.x - m_sizeDelta.cx, point.y - m_sizeDelta.cy);
        m_pWidget->DrawDragImage (&dc, ptDrag);
        m_ptOldImage = ptDrag;
        m_ptPrevPos = point;
    }

    return (dwKeyState & MK_CONTROL) ? DROPEFFECT_COPY : DROPEFFECT_MOVE;
}

void CWidgetView::OnDragLeave() 
{
    CScrollView::OnDragLeave();

    //
    // Erase the old drag image and delete the temporary widget.
    //
    if (m_pWidget != NULL) {
        CClientDC dc (this);
        OnPrepareDC (&dc);
        m_pWidget->DrawDragImage (&dc, m_ptOldImage);
        delete m_pWidget;
        m_pWidget = NULL;
    }
}

BOOL CWidgetView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect,
    CPoint point) 
{
    CScrollView::OnDrop(pDataObject, dropEffect, point);

    //
    // Convert point to logical coordinates.
    //
    CClientDC dc (this);
    OnPrepareDC (&dc);
    dc.DPtoLP (&point);

    //
    // Erase the old drag image and delete the temporary widget.
    //
    if (m_pWidget != NULL) {
        m_pWidget->DrawDragImage (&dc, m_ptOldImage);
        delete m_pWidget;
        m_pWidget = NULL;
    }

    //
    // Retrieve the HGLOBAL from the data object.
    //
    UINT nFormat = ((CWidgetApp*) AfxGetApp ())->m_nFormat;
    HGLOBAL hData = pDataObject->GetGlobalData (nFormat);
    if (hData == NULL)
        return FALSE;

    //
    // Create a widget from the data in the HGLOBAL.
    //
    WIDGETDATA* pWidgetData = (WIDGETDATA*) ::GlobalLock (hData);
    int nType = pWidgetData->nType;
    CRect rect = pWidgetData->rcItem;
    ::GlobalUnlock (hData);

    CRect rcItem;
    rcItem.left = point.x - m_sizeDelta.cx;
    rcItem.top = point.y - m_sizeDelta.cy;
    rcItem.right = rcItem.left + rect.Width ();
    rcItem.bottom = rcItem.top + rect.Height ();

    CWidgetDoc* pDoc = GetDocument ();
    pDoc->AddWidget (nType, &rcItem);
    pDoc->UpdateAllViews (NULL);

    return TRUE;
}

int CWidgetView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
    if (CScrollView::OnCreate(lpCreateStruct) == -1)
        return -1;
    
    m_oleDropTarget.Register (this);
    return 0;
}
